/// <reference types="node" />
import { TextWriter } from '@yellicode/core';
import { RegionMarkerFormatter } from './region-marker-formatter';
/**
 * Internal writer class that is passed to WriterBase implementations. We don't let people inherit from TextWriter directly because
 * we want to control the instantiation and lifetime of each StreamWriter instance.
 */
export declare class StreamWriter implements TextWriter {
    private stream;
    private indent;
    private templateDirName;
    private textFileCache;
    private fileRegionMapper;
    endOfLineString: any;
    indentString: string;
    /**
    * The total number of lines currently written.
    */
    loc: number;
    /**
     * The total number of lines currently written, excluding comments and vertical space.
     */
    sloc: number;
    private countSloc;
    private noIndent;
    private noEndOfLine;
    /**
     * Constructor. Creates a new StreamWriter that writes to the provided WritableStream.
     * @param stream A Node.js WritableStream instance.
     * @param regionMarkerFormatter An optional RegionMarkerFormatter.
     */
    constructor(stream: NodeJS.WritableStream, regionMarkerFormatter?: RegionMarkerFormatter);
    private incrementLoc;
    /******************************************************************************
     *                          TextWriter implementation
    ******************************************************************************/
    /**
    * Writes a new line to the output. The line is indented automatically. The line is ended with the endOfLineString.
    * @param value The line to write. When omitted, only the endOfLineString is written.
    */
    writeLine(value?: string): this;
    /**
    * Writes a collection of lines to the output. Each line is indented automatically and ended with the endOfLineString.
    * @param values The lines to write.
    * @param delimiter An optional delimiter to be written at the end of each line, except for the last one.
    */
    writeLines(values: string[], delimiter?: string): this;
    /**
     * Writes a new line to the output while temporarily increasing the indent. The line is ended with the endOfLineString.
     * @param value The line to write.
     */
    writeLineIndented(value: string): this;
    writeEndOfLine(value?: string): this;
    writeIndent(): this;
    write(value: string): this;
    writeWhiteSpace(): this;
    increaseIndent(): this;
    decreaseIndent(): this;
    clearIndent(): this;
    /**
    * Disables writing any indentation for following writeIndent() and writeLine() calls, until
    * resumeIndent() is called.
    */
    suppressIndent(): this;
    /**
     * Resumes writing indentation after a call to suppressIndent().
     */
    resumeIndent(): this;
    /**
     * Disables writing any end of line character for following and writeLine() calls, until
     * resumeEndOfLine() is called.
     */
    suppressEndOfLine(): this;
    /**
    * Resumes writing end of line characters after a call to suppressEndOfLine().
    */
    resumeEndOfLine(): this;
    writeFile(path: string, encoding?: string): this;
    writeFileRegion(regionName: string, path: string, encoding?: string): boolean;
    /**
   * Prevents the SLOC (source lines of code) counter from being incremented.
   */
    freezeSloc(): this;
    /**
    * Stops preventing the SLOC (source lines of code) counter from being incremented.
    */
    unfreezeSloc(): this;
    /**
     * Writes the contained contents without counting the lines as SLOC (source lines of code).
     * This is similar to calling freezeSloc() -> contents -> unfreezeSloc().
     * @param contents The code to write without increasing SLOC.
     */
    withFrozenSloc(contents: (writer: this) => void): this;
    /******************************************************************************
     *                         End of TextWriter implementation
    ******************************************************************************/
    private createIndentString;
    private resolveFileName;
    private readTextFile;
}
